function  make_lidar_image_uniform_alt(l1b_fName,lat_lim,avg,figureNum,withColorbar)
% make_lidar_image
%
% Usage:
%    make_lidar_image_uniform_alt(l1b_fName,start_lat,len,avg,figureNum,withColorbar)
%
% Example: make_lidar_image(beta_P_ll,202,'withColorbar')
%
% Description:
%    Makes an image of the lidar attenuated backscatter.
%
% Inputs: 
%    l1b_fName    - Level 1B calipso data file name
%    lat_lim      - Two element array (1x2) containting the latitude limits
%                   of the region of interest
%    avg          - number of shots to average, 1 for none, 2 for 2, etc.
%    figureNum    - the figure number you would like to plot the image in
%    withColorbar - [optional] use 'withColorbar' to add the color bar
%
% Requirements: lidar_colorbar.m, kathys_lidar_colors.m, uniform_alt.m, readHDF.m
%
% Acknowledgements: 
% This is based on Kathy Powells IDL code to accomplish a similar task. The colorbar function is based
% on the colorbarf function written by Blair Greenan.
%                   
% Author: R. Kuehn, improved version 7/10/2007
 
% Get the color info
[rgb colors_532 color_bar color_bar_labels] = kathys_lidar_colors('useGrayScale');
% The following code was inserted here to show how the colors are assigned to each pixel (data elements
% within the attenuated backscatter profiles).

% Load altitude array and make it uniform spacing, unlike native CALIOP data
metadata = hdfread(l1b_fName, '/metadata', 'Fields', 'Lidar_Data_Altitudes', 'FirstRecord',1 ,'NumRecords',1);
l1_alt=metadata{1};
alt = uniform_alt_2(20,l1_alt);


% Load all the latitudes array from the L1 file
variable = 'Latitude';
[info lat] = readHDF(l1b_fName,variable);
variable = 'Longitude';
[info lon] = readHDF(l1b_fName,variable);

% TODO Print warning if the wanted latitude is > 75 or < 75.

% For Latitude we need to split the regions where the max or min lat is
% reached this will find the index where the northern or southern latitude
% turn-over point is reached.
dlat = diff(lat);
min_d = min(abs(dlat));
foo = abs(dlat) == min_d; % Creates a logical array
iMaxLat = find(foo,1);    % Find the first occurence of zero in the logical array

% Find the correct latitude region and add the offset for the region 
len = length(lat);

% The code assumes that the region to plot is somewhere in larger segment
% of the orbit segment and is not in the short overlap region
% (latitude-wise) near the poles. We'd need more input information to do
% better, or plot by shot-number.
if iMaxLat > 1e4 % Is the turnover point at the beginning or end of the orbit?
    % It is at the end
    ilat(1) = bsearch(lat(1:iMaxLat),lat_lim(1));
    ilat(2) = bsearch(lat(1:iMaxLat),lat_lim(2));
else
    % It is at the beggining
    ilat(1) = bsearch(lat(iMaxLat:len),lat_lim(1))+iMaxLat;
    ilat(2) = bsearch(lat(iMaxLat:len),lat_lim(2))+iMaxLat;
end

% Find the closest latitude index to lat_lim, yes I'm too lazy to figure out
% the correct indexing.
ix = ilat(1);
d(:,2) = [ix-1;ix;ix+1];
d(1,1) = abs(lat(ix-1)-lat_lim(1));
d(2,1) = abs(lat(ix)-lat_lim(1));
d(3,1) = abs(lat(ix+1)-lat_lim(1));
X = sortrows(d,1);
ilat(1) = X(1,2);

ix = ilat(2);
d(:,2) = [ix-1;ix;ix+1];
d(1,1) = abs(lat(ix-1)-lat_lim(2));
d(2,1) = abs(lat(ix)-lat_lim(2));
d(3,1) = abs(lat(ix+1)-lat_lim(2));
X = sortrows(d,1);
ilat(2) = X(1,2);

fprintf('New latitudes %f %f\n',lat(ilat(1)),lat(ilat(2)));
indA = ilat(1)-1; % DECREMENT for readHDF!1!
indB = (ilat(2)-ilat(1));
% Load attenuated backscatter data
start = [indA 0];
edges = [indB 583];

variable = 'Total_Attenuated_Backscatter_532';
[info betaP_532] = readHDF(l1b_fName,variable,start,edges);
data = betaP_532';
clear betaP_532;

% Average data horizontally
if avg > 1,
    data = avg_lidar_data(data,avg);
end

start = [indA 0];
edges = [indB 1];
variable = 'Latitude';
[info lat2] = readHDF(l1b_fName,variable,start,edges);


B=regrid_lidar(l1_alt,data,alt);




% Make the profile number
%profile_num = (indA+[0:avg:indB-avg]')/15;
profile_num = [1:1:size(B,2)];

% Load profile time data and convert to utc and matlab time format
start = [indA 0];
edges = [indB 1];
variable = 'Profile_Time';
[info TAI] = readHDF(l1b_fName,variable,start,edges);

if avg > 1,
    TAI = avg_lidar_data(TAI,avg);
end

matlab_serial_date = convertTAITime(TAI);
len = length(matlab_serial_date);
datestr([matlab_serial_date(1) matlab_serial_date(len)],21) 

% Load latitude and longitude information
start = [indA 0];
edges = [indB 1];
variable = 'Latitude';
[info lat] = readHDF(l1b_fName,variable,start,edges);

if avg > 1,
    lat = avg_lidar_data(lat,avg);
end

start = [indA 0];
edges = [indB 1];
variable = 'Longitude';
[info lon] = readHDF(l1b_fName,variable,start,edges);

if avg > 1,
    lon = avg_lidar_data(lon,avg);
end

% ist and iend are the selected start and end locations along the orbit track
nAlt = size(B,1);
nProf = size(B,2);


if nargin == 5,
    if strcmp('withColorbar',withColorbar),
	addColorbar = 1;
    end
else
    addColorbar = 0;
end

% For some reason adding the color bar messes up the colormap for the image.
%  I spent a bunch of time trying to figure out why and was unable to determine the exact cause.
% So, when adding a colorbar draw the image as MxNx3
if addColorbar,
    out_img = zeros(nAlt,nProf,3,'uint16');
else
    out = zeros(nAlt,nProf,'uint8');
    out2 = zeros(nAlt,nProf);
end

for ic=1:nProf,
  % the_profile_mfs contains the attenuated backscatter values in units of /km/sr.
  % division by 1.0e-4 converts the attenuated backscatter values to an integer value.
  % each integer value is associated to a color within colors_532.
  tmp = floor(B(:,ic) / 1.0e-4);

  % Filter out the negative data
  neg_mask = (B(:,ic) > 1.0e-4);
  tmp = neg_mask .* tmp + (~neg_mask) * 1;
  % Filter out the data that exceeds 0.1
  over_mask = (B(:,ic) < 0.1);
  tmp = over_mask .* tmp + (~over_mask) * 1001;

  neg_mask = B(:,ic) > 0;
  tmp2 = B(:,ic).*neg_mask + (~neg_mask)*1e-6;
  
  % Convert the out value (which is an index) to a color.
  if addColorbar,
      tmp = colors_532(tmp);
      out_img(:,ic,1) = uint16(rgb(tmp,1)/255*65535);
      out_img(:,ic,2) = uint16(rgb(tmp,2)/255*65535);
      out_img(:,ic,3) = uint16(rgb(tmp,3)/255*65535);
  else
      out(:,ic) = colors_532(tmp) - 1; % Required for uint8 image indexing
      out2(:,ic) = real(log10(tmp2));
  end
end

figure(figureNum)
clf
if addColorbar,
    imgSize = [1024 512];
    TheFig=figure(99);
    temp = get(TheFig,'Position');
    set(TheFig,'Position',[temp(1) temp(2) imgSize(1) imgSize(2)]);
    clf
    h=image(profile_num,alt,out_img);
    IH = gca;
    
    set(gca,'YDir','normal')
    xx = get(gca,'XTick');
    %%cc = cell(size(xx));
    cc = '';
    for j=1:length(xx),
        cc = strvcat(cc,sprintf('%5.2f',lat(xx(j))));
    end
    set(gca,'XTickLabel',cc);
    
    
    cb=lidar_colorbar(rgb,color_bar,color_bar_labels,'vert');
    set(cb,'Position',[0.935 0.113 0.013 0.845])
    
    set(IH,'Position',[0.06 0.111 0.86 0.844]);
    ylabel('Altitude (km)   ')
    xlabel('Latitude   ')
    title('Total Attenuated Backscatter  ')

    
else
    pcolor(profile_num,alt,out2); shading flat;% view(2)
    caxis([-4 -1])
    %colormap(rgb/255);
    %image(profile_num,alt,out);
    %get(h)
    %set(h,'YAxis','reverse')
end
